local moduleName = 'Chat'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0', 'AceEvent-3.0')

module.defaults = {
  profile = {
    enabled = true,

    slashCommands = {
      enabled = true
    },

    messageStyle = {
      enabled = true
    },
  }
}

module.options = {
  name = moduleName,
  desc = 'Options that change the chat frame and chat messages.',
  type = 'group',
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:check()
      end,
    },

    slashCommands = {
      name = 'Slash commands',
      desc = 'Adds slash commands (/rl)',
      type = 'toggle',
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.slashCommands.enabled
      end,
      set = function(_, value)
        module.db.profile.slashCommands.enabled = value
        module.slashCommands:check()
      end
    },

    messageStyle = {
      name = 'Style',
      desc = 'Change the style of your chat messages to a minimal format.',
      type = 'toggle',
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.messageStyle.enabled
      end,
      set = function(_, value)
        module.db.profile.messageStyle.enabled = value
        module.messageStyle:check()
      end
    },
  }
}

function module:enable()
  module.slashCommands:check()
  module.messageStyle:check()
end

function module:disable()
  module.slashCommands:disable()
  module.messageStyle:disable()
end

module.slashCommands = {}

function module.slashCommands:check()
  if module.db.profile.slashCommands.enabled == true then
    self:enable()
  else
    self:disable()
  end
end

-- TODO: Add a slash command /tt that sends a tell to your target.
function module.slashCommands:enable()
  -- TODO: Add a slash command /lfg to open up the group finder.
  SlashCmdList['IDTWEAKS_RELOAD'] = ReloadUI
  SLASH_IDTWEAKS_RELOAD1 = '/rl'
  SLASH_IDTWEAKS_RELOAD2 = '/reload'
end

function module.slashCommands:disable()
  SlashCmdList['IDTWEAKS_RELOAD'] = nil
  SLASH_IDTWEAKS_RELOAD1 = nil
  SLASH_IDTWEAKS_RELOAD2 = nil
end

module.messageStyle = {
  filters = {
    CHAT_MSG_AFK = {
    },
    CHAT_MSG_COMBAT_FACTION_CHANGE = {
      ['Reputation with (.+) increased by (.+).'] = '+ %2 rep (%1)',
    },
    CHAT_MSG_COMBAT_HONOR_GAIN = {
    },
    CHAT_MSG_COMBAT_MISC_INFO = {
    },
    CHAT_MSG_CURRENCY = {
    },
    CHAT_MSG_DND = {
    },
    CHAT_MSG_GUILD_ACHIEVEMENT = {
      ['%%s has earned the achievement (.+)!'] = '\124cff00ff00g %1 %%s\124h',
    },
    CHAT_MSG_IGNORED = {
    },
    CHAT_MSG_LOOT = {
      ['You receive item: (.+)%.'] = '+ %1',
      ['You receive loot: (.+)%.'] = '+ %1',
      ['Received item: (.+)%.'] = '+ %1',
    },
    CHAT_MSG_MONEY = {
      ['Received (.+) Copper'] = '+ %1 c',
      ['Received (.+) Silver'] = '+ %1 s',
      ['Received (.+) Silver, (.+) Copper'] = '+ %1 s %2c',
      ['You loot (.+) Copper'] = '+ %1 c',
      ['You loot (.+) Silver'] = '+ %1 s',
    },
    CHAT_MSG_SKILL = {
      ['Your skill in (.+) has increased to (.+).'] = '+ %2 (%1)'
    },
    CHAT_MSG_SYSTEM = {
      ['Received (.+) Silver'] = '+ %1 s',
      ['Received (.+) of item: (.+).'] = '+ %2x%1',
      ['(.+) completed.'] = '- quest (%1)',
      ['Quest accepted: (.+)'] = '+ quest (%1)',
      ['Received item: (.+)%.'] = '+ %1',
      ['Experience gained: (.+).'] = '+ %1 xp',
      ['(.+) has come online.'] = '+ %1',
      ['(.+) has gone offline.'] = '- %1',
      ['You are now Busy: in combat'] = '+ Combat',
      ['You are no longer marked Busy.'] = '- Combat',
      ['Discovered (.+): (.+) experience gained'] = '+ %2 xp (%1)',
      ['Quest Accepted (.+)'] = '+ quest (%1)',
      ['You are now Away: AFK'] = '+ AFK',
      ['You are no longer Away.'] = '- AFK',
      ['You are no longer rested.'] = '- Rested',
      ['You don\'t meet the requirements for that quest.'] = '! quest (Requirements not met)',
    },
    CHAT_MSG_TARGETICONS = {
    },
    CHAT_MSG_TRADESKILLS = {
    },
  }
}

function module.messageStyle:check()
  if module.db.profile.messageStyle.enabled == true then
    self:enable()
  else
    self:disable()
  end
end

function module.messageStyle:enable()
  for event, eventFilters in pairs(module.messageStyle.filters) do
    ChatFrame_AddMessageEventFilter(event, function(frame, ev, message, ...)
      for k,v in pairs(eventFilters) do
        if message:match(k) then
          message = message:gsub(k, v)
          return nil, message, ...
        end
      end
      return nil, message, ...
    end)
  end
end

function module.messageStyle:disable()
end
